using ShoopNCook.Views;
using System.Windows.Input;
using Models;
using LocalEndpoint;
using Endpoint;

namespace ShoopNCook.Pages;

public partial class RecipePage : ContentPage
{
    private readonly IAccountRecipesPreferences preferences;
    private readonly IUserNotifier notifier;

    public Recipe Recipe { get; init; }

    private uint note;
    private bool isFavorite;


    public ICommand StarCommand => new Command<string>(count => SetNote(uint.Parse(count)));

    public RecipePage(Recipe recipe, IUserNotifier notifier, IAccountRecipesPreferences preferences, uint amount)
    {
        Recipe = recipe;
        BindingContext = this;
        
        InitializeComponent();

        this.preferences = preferences;
        this.notifier = notifier;

        RecipeRate rate = preferences.GetRate(recipe.Info);

        note = rate.Rate;
        isFavorite = rate.IsFavorite;

        SetFavorite(isFavorite);
        SetNote(note);

        Counter.Count = amount;

        if (recipe.Info.Image != null)
            RecipeImage.Source = ImageSource.FromUri(recipe.Info.Image);

        foreach (Ingredient ingredient in recipe.Ingredients)
            IngredientList.Add(new IngredientView(ingredient));

        //retrieves the app's styles
        var styles = Application.Current.Resources.MergedDictionaries.ElementAt(1);

        int count = 0;
        foreach (PreparationStep step in recipe.Steps) {
            //TODO display name of PreparationSteps.
            Label label = new Label();
            label.Style = (Style)styles["Small"];
            label.Text = "Step " + ++count + ": " + step.Description;
            StepList.Add(label);
        }
    }

    private void SetNote(uint note)
    {
        this.note = note;
        int i = 1;
        foreach (ImageButton img in Stars.Children)
        {
            if (i <= note)
            {
                img.Source = ImageSource.FromFile("star_full.svg");
                i++;
            }
            else
                img.Source = ImageSource.FromFile("star_empty.svg");
        }
    }

    private void OnFavorite(object o, EventArgs e)
    {
        SetFavorite(!isFavorite);
        if (isFavorite)
            preferences.AddToFavorites(Recipe.Info);
        else
            preferences.RemoveFromFavorites(Recipe.Info);
    }

    private void OnSubmitReviewClicked(object o, EventArgs e)
    {
        preferences.SetReviewScore(Recipe.Info, note);
        notifier.Success("Your review has been successfuly submited");
    }

    private void OnAddToMyListClicked(object o, EventArgs e)
    {
        if (!preferences.AddToWeeklyList(Recipe.Info, Counter.Count))
            notifier.Notice("You already added this recipe to you weekly list!");
        else
            notifier.Success("Recipe added to your weekly list.");
    }

    private void SetFavorite(bool isFavorite)
    {
        this.isFavorite = isFavorite;
        if (isFavorite)
            Favorite.Source = ImageSource.FromFile("hearth_on.svg");
        else
            Favorite.Source = ImageSource.FromFile("hearth_off.svg");
    }
    private void OnBackButtonClicked(object sender, EventArgs e)
    {
        Navigation.PopAsync();
    }

}